<?php

namespace Drupal\farm_multitenant\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\group\Entity\Group;
use Drupal\group\Entity\GroupInterface;
use Drupal\organization\Entity\OrganizationInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Handles organization-related routes.
 */
class OrganizationController extends ControllerBase {

  /**
   * Redirects to the Add Member form for the group the organization is related
   * to.
   *
   * @param \Drupal\organization\Entity\OrganizationInterface $organization
   *   The organization entity.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response to the Add Member form or the organization page if
   *   the group is not found.
   */
  public function addMember(OrganizationInterface $organization) {
    $group = $this->getGroupFromOrganization($organization);

    if (!$group instanceof GroupInterface) {
      $this->messenger()->addError($this->t('The related group could not be found.'));
      return new RedirectResponse('/organization/' . $organization->id());
    }

    // Return to the organization members page after adding a member.
    $destination = Url::fromRoute('farm_multitenant.organization.members', [
      'organization' => $organization->id(),
    ])->toString();

    $url = Url::fromRoute('entity.group_relationship.add_form', [
      'group' => $group->id(),
      'plugin_id' => 'group_membership',
    ], [
      'query' => [
        'destination' => $destination,
      ],
    ]);

    return new RedirectResponse($url->toString());
  }

  /**
   * Retrieves the group related to the given organization.
   *
   * @param \Drupal\organization\Entity\OrganizationInterface $organization
   *   The organization entity.
   *
   * @return \Drupal\group\Entity\Group|null
   *   The related group entity or NULL if not found.
   */
  protected function getGroupFromOrganization(OrganizationInterface $organization): ?Group {
    $group_relations = $this->entityTypeManager()
      ->getStorage('group_relationship')
      ->loadByProperties([
        'plugin_id' => 'group_organization:farm',
        'entity_id' => $organization->id(),
      ]);

    if (empty($group_relations)) {
      return NULL;
    }

    // There is only one group relation for each organization.
    $group = reset($group_relations)->getGroup();
    return $group instanceof Group ? $group : NULL;
  }
}
