<?php

namespace Drupal\farm_multitenant\Plugin\views\filter;

Use Drupal\farm_multitenant\Service\GroupHelper;
use Drupal\group\Entity\GroupInterface;
use Drupal\group\Entity\GroupRelationship;
use Drupal\user\Entity\User;
use Drupal\views\Plugin\views\filter\FilterPluginBase;
use Drupal\views\Plugin\views\join\JoinPluginBase;
use Drupal\views\Plugin\views\join\Standard;
use Drupal\views\ViewExecutable;
use Drupal\views\Plugin\views\display\DisplayPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Filters content based on the user's group membership. A user can access logs
 * assigned to assets that are assigned to the farm organization associated with
 * the groups that the user is a member of.
 *
 * @ViewsFilter("farm_multitenant_log_access_filter")
 */
class LogAccessFilter extends FilterPluginBase {

  /**
   * The title for the filter.
   *
   * @var string
   */
  protected $filterTitle;

  /**
   * The group helper service.
   *
   * @var \Drupal\farm_multitenant\Service\GroupHelper
   */
  protected $groupHelper;

  public function __construct(array $configuration, $plugin_id, $plugin_definition, GroupHelper $group_helper) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->groupHelper = $group_helper;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('farm_multitenant.group_helper')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function init(ViewExecutable $view, DisplayPluginBase $display, array &$options = NULL) {
    parent::init($view, $display, $options);
    $this->filterTitle = t('Farm log access filter');
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    $account = \Drupal::currentUser();

    // If user can administer logs, don't apply any filter because admins
    // should be able to see logs that are not assigned to a farm.
    if ($account->hasPermission('administer logs')) {
      return;
    }

    $user = User::load($account->id());
    $group_membership_service = \Drupal::service('group.membership_loader');
    $group_memberships = $group_membership_service->loadByUser($user);

    $farm_ids = [];
    foreach ($group_memberships as $group_membership) {
      $group = $group_membership->getGroup();
      $farm_ids = array_merge($farm_ids, $this->groupHelper->getGroupFarmIds($group));
    }

    $farm_ids = array_unique($farm_ids);

    if (empty($farm_ids)) {
      // If no farms found, show nothing
      $this->query->addWhereExpression(0, '1 = 0');
      return;
    }

    // Ensure the log__asset table is part of the query.
    if (!$this->query->ensureTable('log__asset')) {
      $this->query->addWhereExpression(0, '1 = 0');
      return;
    }

    // Get the join manager service
    $join_manager = \Drupal::service('plugin.manager.views.join');

    // Configure the join
    $configuration = [
      'type' => 'INNER',
      'table' => 'asset__farm',
      'field' => 'entity_id',
      'left_table' => 'log__asset',
      'left_field' => 'asset_target_id',
      'operator' => '=',
    ];

    // Create the join plugin instance properly
    $join = $join_manager->createInstance('standard', $configuration);

    // Add the relationship
    $this->query->addRelationship('asset__farm', $join, 'log__asset');

    // Now check if the table is available
    if (!$this->query->ensureTable('asset__farm')) {
      $this->query->addWhereExpression(0, '1 = 0');
      return;
    }

    $this->query->addWhere(0, 'asset__farm.farm_target_id', $farm_ids, 'IN');
  }
}
